###############################################################################
###############################################################################
##                                Opciones.tcl
###############################################################################
###############################################################################
## Creates a dialog in which to choose the options for the program
###############################################################################
###############################################################################
## (c) 2001 Andrs Garca Garca. fandom@retemail.es
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval Opciones {

###############################################################################
# UpdateSpinBox
#    Activates o deactivates the spinboxes.
#
# Parameter:
#    which: The spinbox to update
###############################################################################
proc UpdateSpinBox {which} {
    variable window
    variable optionsTemp

    if {$which=="internal"} {
        if {$optionsTemp(limitLevels)==1} {
            CuadroSpin::EnableSpin $window(intSpin) active
        } else {
            CuadroSpin::EnableSpin $window(intSpin) disable
        }
    } else {
        if {$optionsTemp(external)==1} {
            CuadroSpin::EnableSpin $window(extSpin) active
        } else {
            CuadroSpin::EnableSpin $window(extSpin) disable
        }
    }
    return
}

###############################################################################
# ChangeFilter
#    Invokes the dialog to choose the filter for the download.
###############################################################################
proc ChangeFilter {} {
    global   options
    variable optionsTemp

    set temp                $options(filter)

    Herramientas::FilterFiles

    set optionsTemp(filter) $options(filter)
    set options(Filter)     $temp

    return
}

###############################################################################
# ChooseOptionsWindow
#    Creates the dialog in which to select the different options.
###############################################################################
proc ChooseOptionsWindow {} {
    global labelTitles labelButtons labelDialogs labelFrames labelMenus
    variable optionsTemp
    variable done
    variable window
    global options

    if [winfo exists .optionsWin] {
        return
    }

    set win [toplevel .optionsWin]
    wm resizable $win 0 0
    wm title $win $labelTitles(optionsWin)

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]
    wm geometry $win +[expr $coord(x)+100]+[expr $coord(y)+15]

    set extFrame  [frame $win.extFrame]
    set intFrame  [frame $extFrame.intFrame]
    set butFrame  [frame $extFrame.butFrame]

    set linkFrame [fl::FrameLabel $intFrame.links -bd 2 -relief groove         \
            -textvariable labelFrames(links)]
    set intLinks    [frame $linkFrame.int]
    set updirLinks  [checkbutton $intLinks.updirLinks      \
        -variable Opciones::optionsTemp(dir) -textvariable labelMenus(uplinks)]
    set updateLinks [checkbutton $intLinks.updateLinks     \
        -variable Opciones::optionsTemp(update) -textvariable labelMenus(update)]
    set cgiLinks    [checkbutton $intLinks.cgiLinks        \
        -variable Opciones::optionsTemp(cgi) -textvariable labelMenus(cgi)]

    set levelFrame [fl::FrameLabel $intFrame.levels -bd 2 -relief groove     \
            -textvariable labelFrames(levels)]
    set intLevels  [frame $levelFrame.int]
    set internal   [checkbutton $intLevels.internal  \
        -variable Opciones::optionsTemp(limitLevels) \
        -textvariable labelMenus(levels)             \
        -command "Opciones::UpdateSpinBox internal"]
    set external   [checkbutton $intLevels.external  \
        -variable Opciones::optionsTemp(external)    \
        -textvariable labelMenus(external)           \
        -command "Opciones::UpdateSpinBox external"]
    set intSpin [CuadroSpin::CuadroSpin $intLevels.intspin 0 0 5]
    set extSpin [CuadroSpin::CuadroSpin $intLevels.extspin 0 0 3]
    if {$options(levels)==-1} {
        CuadroSpin::EnableSpin $intSpin disable
    } else {
        CuadroSpin::EnableSpin $intSpin active
        set CuadroSpin::CuadroSpins($intSpin) $optionsTemp(levels)
    }
    if {$optionsTemp(external)==1} {
        CuadroSpin::EnableSpin $extSpin active
        set CuadroSpin::CuadroSpins($extSpin) $optionsTemp(exLevels)
    } else {
        CuadroSpin::EnableSpin $extSpin disable
    }

    set imageFrame   [fl::FrameLabel $intFrame.image   -bd 2 -relief groove  \
            -textvariable labelFrames(images)]
    set intImages    [frame $imageFrame.int]
    set all          [radiobutton $intImages.all        -value 0             \
            -textvariable labelMenus(allImages)                              \
            -variable Opciones::optionsTemp(images)]
    set onlyThumbs   [radiobutton $intImages.onlyThumbs -value 1             \
            -variable Opciones::optionsTemp(images)                          \
            -textvariable labelMenus(onlyThumb)]
    set noThumbs     [radiobutton $intImages.noThumbs   -value 2             \
            -variable Opciones::optionsTemp(images)                          \
            -textvariable labelMenus(noThumb)]
    
    set filterFrame  [fl::FrameLabel $intFrame.filters -bd 2 -relief groove  \
            -textvariable labelFrames(filters)]
    set intFilters   [frame $filterFrame.int]
    set onlyHtml     [checkbutton $intFilters.onlyHtml \
            -variable Opciones::optionsTemp(onlyHtml)  \
            -textvariable labelMenus(onlyHtml)]
    set chooseFilter [button $intFilters.choose        \
            -textvariable labelButtons(chooseFilter)   \
            -command "Opciones::ChangeFilter"]

    set accept [button $butFrame.accept  -textvariable labelButtons(ok)      \
		-width 8 -command "Opciones::ChooseOptionsControl"]
    set cancel [button $butFrame.cancel -textvariable labelButtons(cancel)   \
		-width 8 -command "set Opciones::done 0"]

    set window(win) $win
    set window(intSpin) $intSpin
    set window(extSpin) $extSpin

    grid $extFrame -padx 5 -ipady 5
    grid $intFrame -sticky s

    grid $linkFrame   $levelFrame -padx 5 -pady 5 -sticky news
    grid $filterFrame $imageFrame -padx 5 -pady 5 -sticky news

    grid $intLinks     -sticky w -padx 5 -pady 5 
    grid $updirLinks   -sticky w
    grid $updateLinks  -sticky w
    grid $cgiLinks     -sticky w

    grid $intLevels         -sticky w -padx 5 -pady 5 
    grid $internal $intSpin -sticky w
    grid $external $extSpin -sticky w

    grid $intFilters   -sticky w -padx 5 -pady 5  
    grid $onlyHtml     -sticky w
    grid $chooseFilter  

    grid $intImages    -sticky w -padx 5 -pady 5
    grid $all          -sticky w
    grid $onlyThumbs   -sticky w
    grid $noThumbs     -sticky w

    grid $butFrame -sticky e -padx 2
    grid $accept $cancel -padx 3 -sticky n

    focus $win
    bind $win <Escape> "set Opciones::done 0"

    return
}

###############################################################################
# ChooseOptionsControl
#    Makes sure that the options in the dialog make sense before closing.
###############################################################################
proc ChooseOptionsControl {} {
    global labelTitles labelMessages
    variable window
    variable done
    variable optionsTemp

    if {$optionsTemp(limitLevels)==1} {
        set intLimit [CuadroSpin::GetValue $window(intSpin)]
        if {![regexp {^[0-9]+$} $intLimit]} {
            tk_messageBox -title $labelTitles(error) -parent $window(win) \
                    -message $labelMessages(intNaN) -type ok 
            return
        }
        if {$intLimit<0} {
            CuadroSpin::SetValue $window(intSpin) 0
        }
        if {$intLimit>5} {
            CuadroSpin::SetValue $window(intSpin) 5
        }
    }

    if {$optionsTemp(external)!=0} {
        set extLimit [CuadroSpin::GetValue $window(extSpin)]
        if {![regexp {^[0-9]+$} $extLimit]} {
            tk_messageBox -title $labelTitles(error) -parent $window(win) \
                    -message $labelMessages(extNaN)  -type ok 
            return
        }
        if {$extLimit<0} {
            CuadroSpin::SetValue $window(intSpin) 0
        }
        if {$extLimit>3} {
            CuadroSpin::SetValue $window(intSpin) 3
        }
    }

    set done 1

    return
}

###############################################################################
# ChooseOptions
#    Selects the different options.
###############################################################################
proc ChooseOptions {} {
    global   options
    variable optionsTemp
    variable window
    variable done

    catch {unset optionsTemp}

    foreach {key value} [array get options] {
        set optionsTemp($key) $value
    }

    if {$optionsTemp(levels)==-1} {
        set optionsTemp(limitLevels) 0
    } else {
        set optionsTemp(limitLevels) 1
    }

    ChooseOptionsWindow
    tkwait variable Opciones::done

    if {$done==1} {
        foreach {key value} [array get optionsTemp] {
            set ::options($key) $value
        }
        if {$optionsTemp(limitLevels)==0} {
            set options(levels) -1
        } else {
            set options(levels) [CuadroSpin::GetValue $window(intSpin)]
        }
        if {$optionsTemp(external)==0} {
            set options(exLevels) 0
        } else {
            set options(exLevels) [CuadroSpin::GetValue $window(extSpin)]
        }
    }
    catch {destroy $window(win)}

    return
}
}
