##############################################################################
##############################################################################
#                               Rizo.tcl
##############################################################################
##############################################################################
# In this file are implemented the procedures to actually do the downloading
# by executing 'curl'.
##############################################################################
##############################################################################
# Copyright 1999-2000 Andrs Garca Garca  -- fandom@retemail.es
# Distributed under the terms of the GPL v2
##############################################################################
##############################################################################
namespace eval Rizo {

##############################################################################
# Common
# In version 6.4 and then again in version 6.5 curl changed the format of its
# report, this kludge deals with it.
##############################################################################
proc SetCurlVersion {} {
    global tcl_platform
    variable leftIndex
    variable speedIndex
    variable cookieJar

    catch {exec curl -V} curlVersion
    set version [lindex $curlVersion 1]
    regexp {([0-9]+\.[0-9]+)((?:\.)([0-9]+))?} $curlVersion nada curlVersion nada curlMinor
    set curlVersion $curlVersion$curlMinor
    if {$curlVersion<6.4} {
        set leftIndex  4
        set speedIndex 6
    } elseif {$curlVersion<6.5} {
        set leftIndex  6
        set speedIndex 7
    } else {
        set leftIndex  10
        set speedIndex 11
    }
    if [regexp {Windows} $tcl_platform(os)] {
        if {$curlVersion<7.52} {
            tk_messageBox -title Error -type ok \
                    -message "Your 'cURL' version is too old, please\n\
                              use the one included in the 'win.exe' file: $curlVersion"
            exit
        }
    }

    set cookieJar [file join $::dirGetleft(conf) cookies.txt]

    return
}

##############################################################################
# Common
#    This procedure takes care of initializing the state variables an invoke
#    'curl' for all the connection types.
#
# Parameters:
#    args: args that will be passed to curl
#    type: type of connection:
#           - cab: Headers
#           - dat: The link itself
##############################################################################
proc Common {args type} {
    global options errorCode tcl_platform
    variable curlReport
    variable meta
    variable pipe
    variable curlError
    variable errorMessage
    variable cookieJar

    set curlReport(long)     0
    set curlReport(pause)    0
    set curlReport(speed)    0
    set curlReport(stop)     0
    set curlReport(gotBytes) 0
    set curlError            0
    set errorCode           ""
    set errorMessage        ""
    set ::options(cancelDown) 0

    set args [concat  $args]

    set newArgs [concat {-A "Mozilla/4.0 (compatible; Getleft 0.10.1)"} \
                        -b $cookieJar $args]
    
    if {$options(proxy)==1} {
        if [regexp -nocase {(http://)} $newArgs] {
            set proxy $options(httpProxy)
        } else {
            set proxy $options(ftpProxy)
        }
        set curlCmd [concat curl -x $proxy  $newArgs]
    } else {
        set curlCmd [concat curl $newArgs]
    }
    if {$::DEBUG==1} {
#        puts $curlCmd
    }
    if [regexp {Windows} $tcl_platform(os)] {
        set curlCmd [concat $curlCmd --stderr -]
        eval {set pipe [open "| $curlCmd" r]}
    } else {
        eval {set pipe [open "| $curlCmd 2>@ stdout" r]}
    }

    fileevent $pipe readable [list [namespace code Lector] $pipe $type]
    fconfigure $pipe -blocking 0

    return
}

###############################################################################
# HeadRequest
#    Asks the server for the Headers of the link
#
# Parameters:
#    link: url to download
#    mother: referer page of the link
###############################################################################
proc HeadRequest {link mother} {
    variable meta
# It seems some servers return the body on a HEAD request
    variable workaround

    set meta(content)       ""
    set meta(relocate)      ""
    set meta(charSet)       ""
    set meta(versionServer) ""
    set meta(totalBytes)    -1
    set workaround           0

    if {$mother!="-"} {
        set args [list -e $mother -I $link]
    } else {
        set args [list -I $link]
    }
    Common $args cab

    return
}

###############################################################################
# DataRequest
#   Downloads a given url
#
# Parameters:
#   file:  file in which the url will be saved
#   linke: url to download
#   mother: referer url of the link
###############################################################################
proc DataRequest {file link mother} {
    variable curlReport
    variable meta
    global options

    set curlReport(percentage) 0

    if {$mother!="-"} {
        set args [list -e $mother -o $file $link]
    } else {
        set args [list -o $file $link]
    }

    Common $args dat

    return
}

###############################################################################
# ResumeRequest
#    Resumes, server allowing, a download
#
# Parameters:
#    file: full path of the file where the url will be downloaded
#    link: url to download
#    mother: referer page of the url
###############################################################################
proc ResumeRequest {file link mother} {
    variable curlReport
    variable meta

    set curlReport(percentage) 0

    if {$mother!="-"} {
        set refererUrl $mother
        set args [list -e $refererUrl -c -o $file $link]
    } else {
        set args [list -c -o $file $link]
    }
    Common $args dat

    return
}

###############################################################################
# Lector
#   This procedure controls the downloading, it is invoked anytime there is
#   something to proccess
#
# Parameters:
#   pipe: pipe used to invoke 'curl'
#   tipo: type of request (HEAD, GET, ...) or stopNow, to stop
###############################################################################
proc Lector {pipe tipo} {
    variable meta
    variable curlReport
    variable curlError
    variable workaround
    variable speedIndex
    variable leftIndex
    variable curlVersion
    variable errorMessage
    variable setCookie
    global errorCode getleftState

    if {($tipo=="stopNow")||($getleftState(downloading)==0)} {
        catch {close $pipe}
        return
    }
    if {[catch "eof $pipe" endOfFile]} {
        tk_messageBox -title "Error" -messageBox "Could you please e-mail me\n\
                saying you saw this?" -type ok
        return
    }
    if {($endOfFile) || ($curlReport(stop)==1) || ($curlReport(pause)==1)} {
        if {($endOfFile)} {
            set curlReport(end) 1
        }
        fconfigure $pipe -blocking 1
        if {[catch {close $pipe}]} {
            set curlError [lindex $errorCode 2]
            if {$::DEBUG==1} {
                if {$curlError!=""} {
                    puts "Cdigo de error: $curlError - $errorCode"
                }
            }
	        if {($curlError==18)&&($tipo=="cab")} {
                set curlError 0
                set errorCode ""
            }
        }
        return
    }

    if {[gets $pipe line]>=0} {
        if [string match $line ""] return
        if [regexp {(?:curl: \([0-9]+\) )(.*)} $line nada msg] {
            set errorMessage $msg
            return
        }
if {[regexp {curl} $line]} {
    if {$::DEBUG==1} {
        tk_messageBox -type ok -title Error -message "En rizo: $line"
    }
    return
}
        if {$::DEBUG==1} {
            if {$tipo=="cab"} {
	            if {![regexp {^\s|^1} $line]} {
                    puts $line
		        }
            }
        }
        if {$tipo=="cab"} {
            if {([regexp {^<} $line])||($workaround==1)} {
                set workaround 1
                return
            }
            if [regexp -nocase {^(?:HTTP/)([0-9].[0-9])(?:\ )([0-9]*)(?:\ )(.*)} \
                    $line nada meta(versionServer) meta(code) meta(error)] {
                if {$meta(code)>=400} {
                    set errorMessage $meta(error)
                    catch {error "Server Error" SERVER \
                            "Server \"$meta(error)\" $meta(code)"}
                    return
                }
            }
            regexp -nocase {^(Server: )(.*)}            \
                    $line meta(server)	        
            regexp -nocase {^(?:Location: )(.*)}        \
                    $line nada meta(relocate)
            regexp -nocase {^(?:Content-Type: )([^;]*)} \
                    $line nada meta(content)
            regexp -nocase {^(?:Last-Modified: )(.*)}   \
                    $line nada meta(lastModified)
            regexp -nocase {^(?:Content-Length: )(.*)}  \
                    $line nada meta(totalBytes)
            regexp -nocase {(?:charset=)(.*)} $line nada meta(charSet)
            if [regexp -nocase {Set-Cookie} $line] {
                Cookies::SaveCookie $line
            }
        } else {
            set curlReport(speed) 0
            set curlReport(gotBytes)  0

            if {[string first % $line]!=-1} {
                set curlReport(long) 1
                return
            }
            if {$curlReport(long)==1} {
                set curlReport(speed)  [lindex $line $speedIndex]
                if {$curlReport(speed)==""} return
                set curlReport(percentage) [lindex $line 0]
            } else {
                regexp {(^.*?)(?: )(?:.*\()(.*)( )} $line \
                        nada curlReport(gotBytes) curlReport(speed)
                if [catch {expr $curlReport(gotBytes)*100.0/$meta(totalBytes)} \
                        curlReport(percentage)] {
                    set errorCode ""
                    set curlReport(percentage) 0
                } else {
                    set curlReport(percentage) [expr round($curlReport(percentage))]
                }
                if {$curlReport(gotBytes)<512} {
                    set curlReport(gotBytes) \
                        "[format "%.0f" $curlReport(gotBytes)] bytes"
                } elseif {$curlReport(gotBytes)<1048576} {
                    set curlReport(gotBytes) \
                        "[format "%.2f" [expr $curlReport(gotBytes)/1024.0]] k"
                } else {
                    set curlReport(gotBytes) \
                        "[format "%.2f" [expr $curlReport(gotBytes)/1048576.0]] M"
                }
            }
            if [regexp {k$} $curlReport(speed)] {
                set curlReport(speed) $curlReport(speed)/s
            } else {
                if {![regexp {/} $curlReport(speed)]} {
                    if {($curlReport(speed)>512)} {
                        set curlReport(speed) \
                            "[format "%.2f" [expr $curlReport(speed)/1024.0]] k/s"
                    } else {
                        set curlReport(speed) \
                            "[format "%.0f" $curlReport(speed)] bytes/s"
                    }
                }
            }
            if {$curlReport(long)==1} {
                set curlReport(left) "[lindex $line $leftIndex] \
                        ( $curlReport(speed) )"
            } else {
                set curlReport(left) "$curlReport(gotBytes)\
                        ( $curlReport(speed) )"
            }
        }
    }
    return
}

}
